use std::io::{BufRead, BufReader, Write};
use std::net::{TcpListener, TcpStream};
use std::thread;

pub fn start_tcp_command_server() {
    let listener = TcpListener::bind("127.0.0.1:9632").unwrap();
    println!("listening on port 9632 for tcp commands");

    for stream in listener.incoming() {
        match stream {
            Ok(stream) => {
                thread::spawn(move || handle_client(&stream));
            }
            Err(e) => {
                eprintln!("err with tcp conn: {}", e);
            }
        }
    }
}

fn handle_client(mut stream: &TcpStream) {
    let reader = BufReader::new(stream);

    for line in reader.lines() {
        let buffer = match line {
            Ok(data) => data,
            Err(_) => return,
        };

        if buffer.is_empty() {
            break;
        }

        let mut parts = buffer.trim().split_whitespace();
        if let Some(command) = parts.next() {
            let arguments: Vec<&str> = parts.collect();
            let response = handle_command(command, arguments);
            stream.write_all(response.as_bytes()).unwrap();
        } else {
            stream.write_all("invalid command".as_bytes()).unwrap();
        }
    }
}

fn handle_command(command: &str, arguments: Vec<&str>) -> String {
    match command {
        "ban" => {
            if let (Some(ip), Some(port)) = (arguments.get(0), arguments.get(1)) {
                let iptables = iptables::new(false).unwrap();
                let _ = iptables.append_unique(
                    "filter",
                    "INPUT",
                    &format!("-s {} -p tcp --dport {} -j ACCEPT", ip, port),
                );

                let _ = iptables.append_unique(
                    "filter",
                    "INPUT",
                    &format!("-p tcp --dport {} -j DROP", port),
                );

                format!("banned port {}, only {} allowed", port, ip)
            } else {
                "missing args for ban: ip and port".to_string()
            }
        }
        "unban" => {
            if let (Some(ip), Some(port)) = (arguments.get(0), arguments.get(1)) {
                let iptables = iptables::new(false).unwrap();
                let _ = iptables.delete(
                    "filter",
                    "INPUT",
                    &format!("-s {} -p tcp --dport {} -j ACCEPT", ip, port),
                );

                let _ = iptables.delete(
                    "filter",
                    "INPUT",
                    &format!("-p tcp --dport {} -j DROP", port),
                );

                format!("unbanned port {}, used for {}", port, ip)
            } else {
                "missing args for unban: ip and port".to_string()
            }
        }
        _ => {
            format!("unknown command: {}", command)
        }
    }
}
